import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;

/**
 * @author <A HREF="mailto:weyer@kagi.com">Steve Weyer</A>
 * Copyright (c) 2001, S. Weyer. All right reserved.
 * @version PersonalJava 1.0
 * <A HREF="http://www.kagi.com/weyer/">Steve's other apps</A>
 *
 * ?? separate 'error' area. restore guesses after help? help button?
*/
public class PFB
  extends Frame
  implements
    ItemListener,    // for Choice
    ActionListener   // for Checkbox, Button
  {
  static final boolean DEBUG = false;
  static final String TITLE = "Pico Fermi Bagels";
  static final int NUMDIGIT_MIN = 2;
  static final int NUMDIGIT_MAX = 6;
  static final String DIGITS = "0123456789";
  static final String GUESS_SEP = ": ";
  static final String RESULT_SEP = "\n";

  private Button newGameButton, giveupButton, helpButton, guessButton;
  private TextField guessField;
  private TextArea resultArea;
  private Checkbox allowZeroCheck;
  private Choice numDigitChoice;

  private Random rand;
  public String answer;

  private boolean allowZero = true;
  private int numDigits = 3;

  public static void main(String[] args) {
    String param, val;
    boolean z = true;
    int d = 3;
    for (int i=0; i < args.length; i=i+2) {
      param = args[i];
      val = args[i+1];
      if (param.equals("allowZero"))
        z = (val.equals("true"));
      else if (param.equals("numDigits")) {
        int rdig = DIGITS.indexOf(val.charAt(0), NUMDIGIT_MIN);
        if (rdig >= NUMDIGIT_MIN && rdig <= NUMDIGIT_MAX)
          d = rdig;
      };
    };
    new PFB(z, d);
  }

  public PFB() {
    this(true, 3);
  }

  public PFB(boolean z, int d) {
    super(TITLE);

    allowZero = z;
    numDigits = d;

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent ev) { System.exit(0); } });
    setLayout(new BorderLayout());

    Panel p = new Panel();
    newGameButton = new Button("New");
    newGameButton.addActionListener(this);
    p.add(newGameButton);

    allowZeroCheck = new Checkbox("Allow Zero", allowZero);
    allowZeroCheck.addItemListener(this);
    p.add(allowZeroCheck);

    p.add(new Label("Digits:"));

    numDigitChoice = new Choice();
    for (int i = NUMDIGIT_MIN; i <= NUMDIGIT_MAX; i++)
      numDigitChoice.addItem(new Integer(i).toString());
    //numDigitChoice.select(String.valueOf(numDigits));
    numDigitChoice.select(numDigits - NUMDIGIT_MIN); // default selection
    numDigitChoice.addItemListener(this);
    p.add(numDigitChoice);
    add(p, BorderLayout.SOUTH);

    p = new Panel();
    guessField = new TextField(NUMDIGIT_MAX);
    p.add(guessField);

    guessButton = new Button("Guess");
    guessButton.addActionListener(this);
    p.add(guessButton);

    helpButton = new Button("Help");
    helpButton.addActionListener(this);
    p.add(helpButton);

    giveupButton = new Button("Give Up");
    giveupButton.addActionListener(this);
    p.add(giveupButton);
    add(p, BorderLayout.NORTH);

    resultArea = new TextArea("", 12, 7 * NUMDIGIT_MAX, TextArea.SCROLLBARS_VERTICAL_ONLY);
    add(resultArea, BorderLayout.CENTER);

    //pack();
    Dimension screenSize = getToolkit().getScreenSize();
    setSize(Math.min(screenSize.width,260), Math.min(screenSize.height,300));

    setVisible(true);

    rand = new Random();
    newGame();
  }

  public void itemStateChanged(ItemEvent ev) {
    Object obj = ev.getSource();
    if (obj == allowZeroCheck) {
      if (DEBUG) System.out.println(allowZeroCheck.getLabel() + ": " + allowZeroCheck.getState());
      allowZero = allowZeroCheck.getState();
      newGame();
    }
    else if (obj == numDigitChoice) {
      if (DEBUG) System.out.println("numDigits: " + numDigitChoice.getSelectedItem());
      numDigits = numDigitChoice.getSelectedIndex() + NUMDIGIT_MIN; //Integer.parseInt(numDigitChoice.getSelectedItem());
      newGame();
    }
    else
      if (DEBUG) System.out.println("ItemEvent: " + ev);
  }

   public void actionPerformed(ActionEvent ev) {
    Object obj = ev.getSource();
    if (obj == newGameButton)
      newGame();

    else if (obj == giveupButton)
      resultArea.insert(answer + GUESS_SEP + "** The Answer **" + RESULT_SEP, 0);

    else if (obj == helpButton) {
      resultArea.setText("");
      for (int i= 0; i < HELP_STR.length; i=i+2)
        resultArea.append(HELP_STR[i] + GUESS_SEP + HELP_STR[i + 1] + RESULT_SEP);
    }

    else if (obj == guessButton) {
      String txt = guessField.getText();
      if (txt.length() == 0) return;
      if (DEBUG) System.out.println("guess: " + txt);
      resultArea.insert(txt + GUESS_SEP + evalGuess(txt) + RESULT_SEP, 0);
    }
    else // return key ?? if (obj instanceof TextField) {
      if (DEBUG) System.out.println("ActionEvent: " + ev);
   }

  private void newGame() {
    resultArea.setText("");
    guessField.setText("");

    answer = "";	// new 'blank' answer
    int z0 = (allowZero) ? 0 : 1;
    int z9 = 9, rdig;
    char rch;

    for (int i = 0; i < numDigits; i++) {
      // generate new digit in range (~0|1 - ~9), non-duplicate
      do {
        //rdig = z0 + rand.nextInt(z9-z0); // *** .nextInt >= 1.2
        rdig = z0 + (int) (rand.nextFloat() * (z9-z0));
        rch = DIGITS.charAt(rdig);  // (String.valueOf(rdig)).charAt(0)
      }
      while (answer.indexOf(rch) >= 0);

      answer += rch; // add new digit

      // squeeze range to elim some dups
      if (rdig == z0)
        z0++;
      else if (rdig == z9)
        z9--;
      };

    if (DEBUG) System.out.println("answer: " + answer);
  }


  public String evalGuess(String guess) {
    if (guess.length() == 0)
	return "";

    int i, fermi = 0, pico = 0, z0 = (allowZero) ? 0 : 1;
    StringBuffer result = new StringBuffer(50);

    // check length of guess
    if (guess.length() != numDigits) {
      result.append(numDigits);
      result.append(" digits only");
    }
    else {
      String dupMsg = (numDigits < 4) ? " duplicate; " : " dup; ";
      String invMsg = (numDigits < 4) ? " invalid; "   : " inv; ";
      // check valid, duplicate, guess digits
      char ch;
      for (i = 0; i < numDigits; i++) {
	ch = guess.charAt(i);
        if (DIGITS.indexOf(ch, z0) < 0) {
          result.append(ch);
          result.append(invMsg);
        }
        else if (guess.indexOf(ch,i+1) > i) {
          result.append(ch);
          result.append(dupMsg);
        }
        else if (answer.charAt(i) == ch)
          fermi++;
        else if (answer.indexOf(ch) >= 0)
          pico++;
      };
    };

  if (result.length() == 0) {
    if (pico==0 && fermi==0)
      result.append("Bagels");
    else {
      for (i=0; i < fermi; i++)
        result.append("Fermi ");
        if (fermi == numDigits)
          result.append("!!!");

        else {
          for (i=0; i < pico; i++)
            result.append("Pico ");
        };
      };
    };

  if (DEBUG) System.out.println("result: " + result.toString());
  return result.toString();
  }

  private static final String[] HELP_STR = {
    "PFB",    "by weyer@kagi.com",
    "Zero",   "is zero a possible digit?",
    "Digits", "number of digits: " + PFB.NUMDIGIT_MIN + '-' + PFB.NUMDIGIT_MAX,
    "Fermi",  "correct digit placed correctly",
    "Pico",   "correct digit placed incorrectly",
    "Bagels", "no digits are correct\n",

    "example","Zero: checked; Digits: 3",
    "376",    "Bagels",
    "914",    "Pico",
    "820",    "Fermi Pico",
    "092",    "Fermi Pico Pico",
    "029",    "Fermi Fermi Fermi!",
  };
}
